@@ -14,15 +14,19 @@
 #include <Quotient/events/roomavatarevent.h>
 #include <Quotient/events/roomcanonicalaliasevent.h>
 #include <Quotient/events/roommemberevent.h>
+#include <Quotient/events/roommessageevent.h>
 #include <Quotient/events/roompowerlevelsevent.h>
 #include <Quotient/events/simplestateevents.h>
 #include <Quotient/events/stickerevent.h>
 #include <Quotient/quotient_common.h>
 
-#include "delegatetype.h"
 #include "eventhandler_logging.h"
+#include "events/locationbeaconevent.h"
 #include "events/pollevent.h"
+#include "events/serveraclevent.h"
+#include "events/widgetevent.h"
 #include "linkpreviewer.h"
+#include "messagecomponenttype.h"
 #include "models/reactionmodel.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
@@ -31,34 +35,10 @@
 
 using namespace Quotient;
 
-const NeoChatRoom *EventHandler::getRoom() const
+EventHandler::EventHandler(const NeoChatRoom *room, const RoomEvent *event)
+    : m_room(room)
+    , m_event(event)
 {
-    return m_room;
-}
-
-void EventHandler::setRoom(const NeoChatRoom *room)
-{
-    if (room == m_room) {
-        return;
-    }
-    m_room = room;
-}
-
-const Quotient::Event *EventHandler::getEvent() const
-{
-    return m_event;
-}
-
-void EventHandler::setEvent(const Quotient::RoomEvent *event)
-{
-    if (m_room == nullptr) {
-        qCWarning(EventHandling) << "cannot setEvent when m_room is set to nullptr.";
-        return;
-    }
-    if (event == m_event) {
-        return;
-    }
-    m_event = event;
 }
 
 QString EventHandler::getId() const
@@ -71,62 +51,14 @@ QString EventHandler::getId() const
     return !m_event->id().isEmpty() ? m_event->id() : m_event->transactionId();
 }
 
-DelegateType::Type EventHandler::getDelegateTypeForEvent(const Quotient::RoomEvent *event) const
-{
-    if (auto e = eventCast<const RoomMessageEvent>(event)) {
-        switch (e->msgtype()) {
-        case MessageEventType::Emote:
-            return DelegateType::Emote;
-        case MessageEventType::Notice:
-            return DelegateType::Notice;
-        case MessageEventType::Image:
-            return DelegateType::Image;
-        case MessageEventType::Audio:
-            return DelegateType::Audio;
-        case MessageEventType::Video:
-            return DelegateType::Video;
-        case MessageEventType::Location:
-            return DelegateType::Location;
-        default:
-            break;
-        }
-        if (e->hasFileContent()) {
-            return DelegateType::File;
-        }
-
-        return DelegateType::Message;
-    }
-    if (is<const StickerEvent>(*event)) {
-        return DelegateType::Sticker;
-    }
-    if (event->isStateEvent()) {
-        if (event->matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
-            return DelegateType::LiveLocation;
-        }
-        return DelegateType::State;
-    }
-    if (is<const EncryptedEvent>(*event)) {
-        return DelegateType::Encrypted;
-    }
-    if (is<PollStartEvent>(*event)) {
-        const auto pollEvent = eventCast<const PollStartEvent>(event);
-        if (pollEvent->isRedacted()) {
-            return DelegateType::Message;
-        }
-        return DelegateType::Poll;
-    }
-
-    return DelegateType::Other;
-}
-
-DelegateType::Type EventHandler::getDelegateType() const
+MessageComponentType::Type EventHandler::messageComponentType() const
 {
     if (m_event == nullptr) {
-        qCWarning(EventHandling) << "getDelegateType called with m_event set to nullptr.";
-        return DelegateType::Other;
+        qCWarning(EventHandling) << "messageComponentType called with m_event set to nullptr.";
+        return MessageComponentType::Other;
     }
 
-    return getDelegateTypeForEvent(m_event);
+    return MessageComponentType::typeForEvent(*m_event);
 }
 
 QVariantMap EventHandler::getAuthor(bool isPending) const
@@ -300,6 +232,15 @@ bool EventHandler::isHidden()
     return false;
 }
 
+Qt::TextFormat EventHandler::messageBodyInputFormat(const Quotient::RoomMessageEvent &event)
+{
+    if (event.mimeType().name() == "text/plain"_ls) {
+        return Qt::PlainText;
+    } else {
+        return Qt::RichText;
+    }
+}
+
 QString EventHandler::rawMessageBody(const Quotient::RoomMessageEvent &event)
 {
     if (event.hasFileContent()) {
@@ -382,8 +323,7 @@ QString EventHandler::getBody(const Quotient::RoomEvent *event, Qt::TextFormat f
             }
 
             if (prettyPrint) {
-                subjectName = QStringLiteral("<a href=\"https://matrix.to/#/%1\" style=\"color: %2\">%3</a>")
-                                  .arg(e.userId(), Utils::getUserColor(m_room->user(e.userId())->hueF()).name(), subjectName);
+                subjectName = QStringLiteral("<a href=\"https://matrix.to/#/%1\">%2</a>").arg(e.userId(), subjectName);
             }
 
             // The below code assumes senderName output in AuthorRole
@@ -494,22 +434,22 @@ QString EventHandler::getBody(const Quotient::RoomEvent *event, Qt::TextFormat f
         [](const RoomPowerLevelsEvent &) {
             return i18nc("'power level' means permission level", "changed the power levels for this room");
         },
+        [](const LocationBeaconEvent &e) {
+            return e.contentJson()["description"_ls].toString();
+        },
+        [](const ServerAclEvent &) {
+            return i18n("changed the server access control lists for this room");
+        },
+        [](const WidgetEvent &e) {
+            if (e.fullJson()["unsigned"_ls]["prev_content"_ls].toObject().isEmpty()) {
+                return i18nc("[User] added <name> widget", "added %1 widget", e.contentJson()["name"_ls].toString());
+            }
+            if (e.contentJson().isEmpty()) {
+                return i18nc("[User] removed <name> widget", "removed %1 widget", e.fullJson()["unsigned"_ls]["prev_content"_ls]["name"_ls].toString());
+            }
+            return i18nc("[User] configured <name> widget", "configured %1 widget", e.contentJson()["name"_ls].toString());
+        },
         [prettyPrint](const StateEvent &e) {
-            if (e.matrixType() == QLatin1String("m.room.server_acl")) {
-                return i18n("changed the server access control lists for this room");
-            }
-            if (e.matrixType() == QLatin1String("im.vector.modular.widgets")) {
-                if (e.fullJson()["unsigned"_ls]["prev_content"_ls].toObject().isEmpty()) {
-                    return i18nc("[User] added <name> widget", "added %1 widget", e.contentJson()["name"_ls].toString());
-                }
-                if (e.contentJson().isEmpty()) {
-                    return i18nc("[User] removed <name> widget", "removed %1 widget", e.fullJson()["unsigned"_ls]["prev_content"_ls]["name"_ls].toString());
-                }
-                return i18nc("[User] configured <name> widget", "configured %1 widget", e.contentJson()["name"_ls].toString());
-            }
-            if (e.matrixType() == "org.matrix.msc3672.beacon_info"_ls) {
-                return e.contentJson()["description"_ls].toString();
-            }
             return e.stateKey().isEmpty() ? i18n("updated %1 state", e.matrixType())
                                           : i18n("updated %1 state for %2", e.matrixType(), prettyPrint ? e.stateKey().toHtmlEscaped() : e.stateKey());
         },
@@ -551,7 +491,7 @@ QString EventHandler::getMessageBody(const RoomMessageEvent &event, Qt::TextForm
     }
 
     if (format == Qt::RichText) {
-        return textHandler.handleRecieveRichText(inputFormat, m_room, &event, stripNewlines);
+        return textHandler.handleRecieveRichText(inputFormat, m_room, &event, stripNewlines, event.isReplaced());
     } else {
         return textHandler.handleRecievePlainText(inputFormat, stripNewlines);
     }
@@ -663,19 +603,22 @@ QString EventHandler::getGenericBody() const
         [](const RoomPowerLevelsEvent &) {
             return i18nc("'power level' means permission level", "changed the power levels for this room");
         },
-        [](const StateEvent &e) {
-            if (e.matrixType() == QLatin1String("m.room.server_acl")) {
-                return i18n("changed the server access control lists for this room");
+        [](const LocationBeaconEvent &) {
+            return i18n("sent a live location beacon");
+        },
+        [](const ServerAclEvent &) {
+            return i18n("changed the server access control lists for this room");
+        },
+        [](const WidgetEvent &e) {
+            if (e.fullJson()["unsigned"_ls]["prev_content"_ls].toObject().isEmpty()) {
+                return i18n("added a widget");
             }
-            if (e.matrixType() == QLatin1String("im.vector.modular.widgets")) {
-                if (e.fullJson()["unsigned"_ls]["prev_content"_ls].toObject().isEmpty()) {
-                    return i18n("added a widget");
-                }
-                if (e.contentJson().isEmpty()) {
-                    return i18n("removed a widget");
-                }
-                return i18n("configured a widget");
+            if (e.contentJson().isEmpty()) {
+                return i18n("removed a widget");
             }
+            return i18n("configured a widget");
+        },
+        [](const StateEvent &) {
             return i18n("updated the state");
         },
         [](const PollStartEvent &e) {
@@ -713,6 +656,7 @@ QVariantMap EventHandler::getMediaInfoForEvent(const Quotient::RoomEvent *event)
 
     // Get the file info for the event.
     const EventContent::FileInfo *fileInfo;
+    bool isSticker = false;
     if (event->is<RoomMessageEvent>()) {
         auto roomMessageEvent = eventCast<const RoomMessageEvent>(event);
         if (!roomMessageEvent->hasFileContent()) {
@@ -722,14 +666,15 @@ QVariantMap EventHandler::getMediaInfoForEvent(const Quotient::RoomEvent *event)
     } else if (event->is<StickerEvent>()) {
         auto stickerEvent = eventCast<const StickerEvent>(event);
         fileInfo = &stickerEvent->image();
+        isSticker = true;
     } else {
         return {};
     }
 
-    return getMediaInfoFromFileInfo(fileInfo, eventId);
+    return getMediaInfoFromFileInfo(fileInfo, eventId, false, isSticker);
 }
 
-QVariantMap EventHandler::getMediaInfoFromFileInfo(const EventContent::FileInfo *fileInfo, const QString &eventId, bool isThumbnail) const
+QVariantMap EventHandler::getMediaInfoFromFileInfo(const EventContent::FileInfo *fileInfo, const QString &eventId, bool isThumbnail, bool isSticker) const
 {
     QVariantMap mediaInfo;
 
@@ -756,6 +701,8 @@ QVariantMap EventHandler::getMediaInfoFromFileInfo(const EventContent::FileInfo
     // Add media size if available.
     mediaInfo["size"_ls] = fileInfo->payloadSize;
 
+    mediaInfo["isSticker"_ls] = isSticker;
+
     // Add parameter depending on media type.
     if (mimeType.name().contains(QStringLiteral("image"))) {
         if (auto castInfo = static_cast<const EventContent::ImageContent *>(fileInfo)) {
@@ -832,22 +779,22 @@ QString EventHandler::getReplyId() const
     return m_event->contentJson()["m.relates_to"_ls].toObject()["m.in_reply_to"_ls].toObject()["event_id"_ls].toString();
 }
 
-DelegateType::Type EventHandler::getReplyDelegateType() const
+MessageComponentType::Type EventHandler::replyMessageComponentType() const
 {
     if (m_room == nullptr) {
-        qCWarning(EventHandling) << "getReplyDelegateType called with m_room set to nullptr.";
-        return DelegateType::Other;
+        qCWarning(EventHandling) << "replyMessageComponentType called with m_room set to nullptr.";
+        return MessageComponentType::Other;
     }
     if (m_event == nullptr) {
-        qCWarning(EventHandling) << "getReplyDelegateType called with m_event set to nullptr.";
-        return DelegateType::Other;
+        qCWarning(EventHandling) << "replyMessageComponentType called with m_event set to nullptr.";
+        return MessageComponentType::Other;
     }
 
     auto replyEvent = m_room->getReplyForEvent(*m_event);
     if (replyEvent == nullptr) {
-        return DelegateType::Other;
+        return MessageComponentType::Other;
     }
-    return getDelegateTypeForEvent(replyEvent);
+    return MessageComponentType::typeForEvent(*replyEvent);
 }
 
 QVariantMap EventHandler::getReplyAuthor() const
