@@ -2,14 +2,14 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include "messageeventmodel.h"
-#include "linkpreviewer.h"
+#include "messagecomponenttype.h"
 #include "messageeventmodel_logging.h"
 
 #include "neochatconfig.h"
 
-#include <Quotient/connection.h>
 #include <Quotient/csapi/rooms.h>
 #include <Quotient/events/redactionevent.h>
+#include <Quotient/events/roommessageevent.h>
 #include <Quotient/events/stickerevent.h>
 #include <Quotient/user.h>
 
@@ -22,6 +22,9 @@
 #include "enums/delegatetype.h"
 #include "eventhandler.h"
 #include "events/pollevent.h"
+#include "linkpreviewer.h"
+#include "messagecontentmodel.h"
+#include "models/messagefiltermodel.h"
 #include "models/reactionmodel.h"
 #include "texthandler.h"
 
@@ -31,7 +34,6 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
 {
     QHash<int, QByteArray> roles = QAbstractItemModel::roleNames();
     roles[DelegateTypeRole] = "delegateType";
-    roles[PlainText] = "plainText";
     roles[EventIdRole] = "eventId";
     roles[TimeRole] = "time";
     roles[TimeStringRole] = "timeString";
@@ -40,18 +42,8 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[HighlightRole] = "isHighlighted";
     roles[SpecialMarksRole] = "marks";
     roles[ProgressInfoRole] = "progressInfo";
-    roles[ShowLinkPreviewRole] = "showLinkPreview";
-    roles[LinkPreviewRole] = "linkPreview";
-    roles[MediaInfoRole] = "mediaInfo";
-    roles[IsReplyRole] = "isReply";
-    roles[ReplyAuthor] = "replyAuthor";
-    roles[ReplyIdRole] = "replyId";
-    roles[ReplyDelegateTypeRole] = "replyDelegateType";
-    roles[ReplyDisplayRole] = "replyDisplay";
-    roles[ReplyMediaInfoRole] = "replyMediaInfo";
     roles[IsThreadedRole] = "isThreaded";
     roles[ThreadRootRole] = "threadRoot";
-    roles[ShowAuthorRole] = "showAuthor";
     roles[ShowSectionRole] = "showSection";
     roles[ReadMarkersRole] = "readMarkers";
     roles[ExcessReadMarkersRole] = "excessReadMarkers";
@@ -64,10 +56,9 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[IsRedactedRole] = "isRedacted";
     roles[GenericDisplayRole] = "genericDisplay";
     roles[IsPendingRole] = "isPending";
-    roles[LatitudeRole] = "latitude";
-    roles[LongitudeRole] = "longitude";
-    roles[AssetRole] = "asset";
-    roles[PollHandlerRole] = "pollHandler";
+    roles[ContentModelRole] = "contentModel";
+    roles[MediaInfoRole] = "mediaInfo";
+    roles[IsEditableRole] = "isEditable";
     return roles;
 }
 
@@ -96,7 +87,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
     beginResetModel();
     if (m_currentRoom) {
         m_currentRoom->disconnect(this);
-        m_linkPreviewers.clear();
         m_reactionModels.clear();
     }
 
@@ -119,14 +109,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             room->getPreviousContent(50);
         }
         lastReadEventId = room->lastFullyReadEventId();
-        connect(m_currentRoom, &NeoChatRoom::replyLoaded, this, [this](const auto &eventId, const auto &replyId) {
-            Q_UNUSED(replyId);
-            auto row = eventIdToRow(eventId);
-            if (row == -1) {
-                return;
-            }
-            Q_EMIT dataChanged(index(row, 0), index(row, 0), {ReplyDelegateTypeRole, ReplyDisplayRole, ReplyMediaInfoRole, ReplyAuthor});
-        });
 
         connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [this](RoomEventsRange events) {
             for (auto &&event : events) {
@@ -134,34 +116,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
 
                 if (message != nullptr) {
                     createEventObjects(message);
-                    if (NeoChatConfig::self()->showFancyEffects()) {
-                        QString planBody = message->plainBody();
-                        // snowflake
-                        const QString snowlakeEmoji = QString::fromUtf8("\xE2\x9D\x84");
-                        if (planBody.contains(snowlakeEmoji)) {
-                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("snowflake"));
-                        }
-                        // fireworks
-                        const QString fireworksEmoji = QString::fromUtf8("\xF0\x9F\x8E\x86");
-                        if (planBody.contains(fireworksEmoji)) {
-                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("fireworks"));
-                        }
-                        // sparkler
-                        const QString sparklerEmoji = QString::fromUtf8("\xF0\x9F\x8E\x87");
-                        if (planBody.contains(sparklerEmoji)) {
-                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("fireworks"));
-                        }
-                        // party pooper
-                        const QString partyEmoji = QString::fromUtf8("\xF0\x9F\x8E\x89");
-                        if (planBody.contains(partyEmoji)) {
-                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("confetti"));
-                        }
-                        // confetti ball
-                        const QString confettiEmoji = QString::fromUtf8("\xF0\x9F\x8E\x8A");
-                        if (planBody.contains(confettiEmoji)) {
-                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("confetti"));
-                        }
-                    }
                 }
                 if (event->is<PollStartEvent>()) {
                     m_currentRoom->createPollHandler(eventCast<const PollStartEvent>(event.get()));
@@ -195,7 +149,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             if (biggest < m_currentRoom->maxTimelineIndex()) {
                 auto rowBelowInserted = m_currentRoom->maxTimelineIndex() - biggest + timelineBaseIndex() - 1;
-                refreshEventRoles(rowBelowInserted, {ShowAuthorRole});
+                refreshEventRoles(rowBelowInserted, {MessageFilterModel::ShowAuthorRole});
             }
             for (auto i = m_currentRoom->maxTimelineIndex() - biggest; i <= m_currentRoom->maxTimelineIndex() - lowest; ++i) {
                 refreshLastUserEvents(i);
@@ -225,7 +179,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             refreshRow(timelineBaseIndex()); // Refresh the looks
             refreshLastUserEvents(0);
             if (timelineBaseIndex() > 0) { // Refresh below, see #312
-                refreshEventRoles(timelineBaseIndex() - 1, {ShowAuthorRole});
+                refreshEventRoles(timelineBaseIndex() - 1, {MessageFilterModel::ShowAuthorRole});
             }
         });
         connect(m_currentRoom, &Room::pendingEventChanged, this, &MessageEventModel::refreshRow);
@@ -238,7 +192,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             moveReadMarker(toEventId);
         });
         connect(m_currentRoom, &Room::replacedEvent, this, [this](const RoomEvent *newEvent) {
-            refreshLastUserEvents(refreshEvent(newEvent->id()) - timelineBaseIndex());
             const RoomMessageEvent *message = eventCast<const RoomMessageEvent>(newEvent);
             if (message != nullptr) {
                 createEventObjects(message);
@@ -265,10 +218,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
                 refreshEventRoles(event->id(), {ReadMarkersRole, ReadMarkersStringRole, ExcessReadMarkersRole});
             }
         });
-        connect(m_currentRoom, &Room::newFileTransfer, this, &MessageEventModel::refreshEvent);
-        connect(m_currentRoom, &Room::fileTransferProgress, this, &MessageEventModel::refreshEvent);
-        connect(m_currentRoom, &Room::fileTransferCompleted, this, &MessageEventModel::refreshEvent);
-        connect(m_currentRoom, &Room::fileTransferFailed, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom->connection(), &Connection::ignoredUsersListChanged, this, [this] {
             beginResetModel();
             endResetModel();
@@ -439,8 +388,6 @@ void MessageEventModel::fetchMore(const QModelIndex &parent)
     }
 }
 
-static LinkPreviewer *emptyLinkPreview = new LinkPreviewer;
-
 QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 {
     if (!checkIndex(idx, QAbstractItemModel::CheckIndexOption::IndexIsValid)) {
@@ -481,9 +428,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     const auto pendingIt = m_currentRoom->pendingEvents().crbegin() + std::min(row, timelineBaseIndex());
     const auto &evt = isPending ? **pendingIt : **timelineIt;
 
-    EventHandler eventHandler;
-    eventHandler.setRoom(m_currentRoom);
-    eventHandler.setEvent(&evt);
+    EventHandler eventHandler(m_currentRoom, &evt);
 
     if (role == Qt::DisplayRole) {
         if (evt.isRedacted()) {
@@ -494,16 +439,24 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return eventHandler.getRichBody();
     }
 
+    if (role == ContentModelRole) {
+        if (!evt.isStateEvent()) {
+            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&evt, m_currentRoom));
+        }
+        if (evt.isStateEvent()) {
+            if (evt.matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
+                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&evt, m_currentRoom));
+            }
+        }
+        return {};
+    }
+
     if (role == GenericDisplayRole) {
         return eventHandler.getGenericBody();
     }
 
-    if (role == PlainText) {
-        return eventHandler.getPlainBody();
-    }
-
     if (role == DelegateTypeRole) {
-        return eventHandler.getDelegateType();
+        return DelegateType::typeForEvent(evt);
     }
 
     if (role == AuthorRole) {
@@ -561,46 +514,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return eventHandler.getTimeString(true, QLocale::ShortFormat, isPending, lastUpdated);
     }
 
-    if (role == ShowLinkPreviewRole) {
-        return m_linkPreviewers.contains(evt.id());
-    }
-
-    if (role == LinkPreviewRole) {
-        if (m_linkPreviewers.contains(evt.id())) {
-            return QVariant::fromValue<LinkPreviewer *>(m_linkPreviewers[evt.id()].data());
-        } else {
-            return QVariant::fromValue<LinkPreviewer *>(emptyLinkPreview);
-        }
-    }
-
-    if (role == MediaInfoRole) {
-        return eventHandler.getMediaInfo();
-    }
-
-    if (role == IsReplyRole) {
-        return eventHandler.hasReply();
-    }
-
-    if (role == ReplyIdRole) {
-        return eventHandler.getReplyId();
-    }
-
-    if (role == ReplyDelegateTypeRole) {
-        return eventHandler.getReplyDelegateType();
-    }
-
-    if (role == ReplyAuthor) {
-        return eventHandler.getReplyAuthor();
-    }
-
-    if (role == ReplyDisplayRole) {
-        return eventHandler.getReplyRichBody();
-    }
-
-    if (role == ReplyMediaInfoRole) {
-        return eventHandler.getReplyMediaInfo();
-    }
-
     if (role == IsThreadedRole) {
         return eventHandler.isThreaded();
     }
@@ -609,25 +522,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return eventHandler.threadRoot();
     }
 
-    if (role == ShowAuthorRole) {
-        for (auto r = row + 1; r < rowCount(); ++r) {
-            auto i = index(r);
-            // Note !itemData(i).empty() is a check for instances where rows have been removed, e.g. when the read marker is moved.
-            // While the row is removed the subsequent row indexes are not changed so we need to skip over the removed index.
-            // See - https://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows
-            if (data(i, SpecialMarksRole) != EventStatus::Hidden && !itemData(i).empty()) {
-                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, DelegateTypeRole) == DelegateType::State
-                    || data(i, TimeRole).toDateTime().msecsTo(data(idx, TimeRole).toDateTime()) > 600000
-                    || data(i, TimeRole).toDateTime().toLocalTime().date().day() != data(idx, TimeRole).toDateTime().toLocalTime().date().day()
-                    // FIXME: This should not be necessary; the proper fix is to calculate this role in MessageFilterModel with the knowledge about the filtered
-                    // events.
-                    || data(i, IsRedactedRole).toBool();
-            }
-        }
-
-        return true;
-    }
-
     if (role == ShowSectionRole) {
         for (auto r = row + 1; r < rowCount(); ++r) {
             auto i = index(r);
@@ -644,18 +538,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return false;
     }
 
-    if (role == LatitudeRole) {
-        return eventHandler.getLatitude();
-    }
-
-    if (role == LongitudeRole) {
-        return eventHandler.getLongitude();
-    }
-
-    if (role == AssetRole) {
-        return eventHandler.getLocationAssetType();
-    }
-
     if (role == ReadMarkersRole) {
         return eventHandler.getReadMarkers();
     }
@@ -705,8 +587,12 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return row < static_cast<int>(m_currentRoom->pendingEvents().size());
     }
 
-    if (role == PollHandlerRole) {
-        return QVariant::fromValue<PollHandler *>(m_currentRoom->poll(evt.id()));
+    if (role == MediaInfoRole) {
+        return eventHandler.getMediaInfo();
+    }
+
+    if (role == IsEditableRole) {
+        return eventHandler.messageComponentType() == MessageComponentType::Text && evt.senderId() == m_currentRoom->localUser()->id();
     }
 
     return {};
@@ -714,6 +600,10 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 
 int MessageEventModel::eventIdToRow(const QString &eventID) const
 {
+    if (m_currentRoom == nullptr) {
+        return -1;
+    }
+
     const auto it = m_currentRoom->findInTimeline(eventID);
     if (it == m_currentRoom->historyEdge()) {
         // qWarning() << "Trying to find inexistent event:" << eventID;
@@ -726,16 +616,6 @@ void MessageEventModel::createEventObjects(const Quotient::RoomMessageEvent *eve
 {
     auto eventId = event->id();
 
-    if (m_linkPreviewers.contains(eventId)) {
-        if (!LinkPreviewer::hasPreviewableLinks(event)) {
-            m_linkPreviewers.remove(eventId);
-        }
-    } else {
-        if (LinkPreviewer::hasPreviewableLinks(event)) {
-            m_linkPreviewers[eventId] = QSharedPointer<LinkPreviewer>(new LinkPreviewer(m_currentRoom, event));
-        }
-    }
-
     // ReactionModel handles updates to add and remove reactions, we only need to
     // handle adding and removing whole models here.
     if (m_reactionModels.contains(eventId)) {
@@ -763,7 +643,7 @@ void MessageEventModel::createEventObjects(const Quotient::RoomMessageEvent *eve
 bool MessageEventModel::event(QEvent *event)
 {
     if (event->type() == QEvent::ApplicationPaletteChange) {
-        Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, 0), {AuthorRole, ReplyAuthor, ReadMarkersRole});
+        Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, 0), {AuthorRole, ReadMarkersRole});
     }
     return QObject::event(event);
 }
