@@ -40,7 +40,6 @@ public:
      */
     enum EventRoles {
         DelegateTypeRole = Qt::UserRole + 1, /**< The delegate type of the message. */
-        PlainText, /**< Plain text representation of the message. */
         EventIdRole, /**< The matrix event ID of the event. */
         TimeRole, /**< The timestamp for when the event was sent (as a QDateTime). */
         TimeStringRole, /**< The timestamp for when the event was sent as a string (in QLocale::ShortFormat). */
@@ -50,23 +49,13 @@ public:
         SpecialMarksRole, /**< Whether the event is hidden or not. */
         ProgressInfoRole, /**< Progress info when downloading files. */
         GenericDisplayRole, /**< A generic string based upon the message type. */
-
-        ShowLinkPreviewRole, /**< Whether a link preview should be shown. */
-        LinkPreviewRole, /**< The link preview details. */
-
         MediaInfoRole, /**< The media info for the event. */
 
-        IsReplyRole, /**< Is the message a reply to another event. */
-        ReplyAuthor, /**< The author of the event that was replied to. */
-        ReplyIdRole, /**< The matrix ID of the message that was replied to. */
-        ReplyDelegateTypeRole, /**< The delegate type of the message that was replied to. */
-        ReplyDisplayRole, /**< The body of the message that was replied to. */
-        ReplyMediaInfoRole, /**< The media info of the message that was replied to. */
+        ContentModelRole, /**< The MessageContentModel for the event. */
 
         IsThreadedRole,
         ThreadRootRole,
 
-        ShowAuthorRole, /**< Whether the author's name should be shown. */
         ShowSectionRole, /**< Whether the section header should be shown. */
 
         ReadMarkersRole, /**< The first 5 other users at the event for read marker tracking. */
@@ -80,10 +69,7 @@ public:
         AuthorDisplayNameRole, /**< The displayname for the event's sender; for name change events, the old displayname. */
         IsRedactedRole, /**< Whether an event has been deleted. */
         IsPendingRole, /**< Whether an event is waiting to be accepted by the server. */
-        LatitudeRole, /**< Latitude for a location event. */
-        LongitudeRole, /**< Longitude for a location event. */
-        AssetRole, /**< Type of location event, e.g. self pin of the user location. */
-        PollHandlerRole, /**< The PollHandler for the event, if any. */
+        IsEditableRole, /**< Whether the event can be edited by the user. */
         LastRole, // Keep this last
     };
     Q_ENUM(EventRoles)
@@ -127,7 +113,7 @@ private Q_SLOTS:
     void refreshRow(int row);
 
 private:
-    NeoChatRoom *m_currentRoom = nullptr;
+    QPointer<NeoChatRoom> m_currentRoom = nullptr;
     QString lastReadEventId;
     QPersistentModelIndex m_lastReadEventIndex;
     int rowBelowInserted = -1;
@@ -135,7 +121,6 @@ private:
     bool movingEvent = false;
     KFormat m_format;
 
-    QMap<QString, QSharedPointer<LinkPreviewer>> m_linkPreviewers;
     QMap<QString, QSharedPointer<ReactionModel>> m_reactionModels;
 
     [[nodiscard]] int timelineBaseIndex() const;
@@ -155,5 +140,4 @@ private:
 
 Q_SIGNALS:
     void roomChanged();
-    void fancyEffectsReasonFound(const QString &fancyEffect);
 };
