@@ -10,6 +10,7 @@
 #include <QTemporaryFile>
 
 #include <Quotient/jobs/basejob.h>
+#include <Quotient/quotient_common.h>
 #include <Quotient/user.h>
 #include <qcoro/qcorosignal.h>
 
@@ -42,6 +43,7 @@
 #include "neochatconfig.h"
 #include "notificationsmanager.h"
 #include "roomlastmessageprovider.h"
+#include "spacehierarchycache.h"
 #include "texthandler.h"
 #include "urlhelper.h"
 #include "utils.h"
@@ -125,6 +127,33 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
             Q_EMIT urlPreviewEnabledChanged();
         }
     });
+    connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceHierarchyChanged, this, [this]() {
+        if (isSpace()) {
+            Q_EMIT childrenNotificationCountChanged();
+            Q_EMIT childrenHaveHighlightNotificationsChanged();
+        }
+    });
+    connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceNotifcationCountChanged, this, [this](const QStringList &spaces) {
+        if (spaces.contains(id())) {
+            Q_EMIT childrenNotificationCountChanged();
+            Q_EMIT childrenHaveHighlightNotificationsChanged();
+        }
+    });
+}
+
+int NeoChatRoom::contextAwareNotificationCount() const
+{
+    // DOn't include spaces, rooms that the user hasn't joined and rooms where the user has joined the successor.
+    if (isSpace() || joinState() != JoinState::Join || successor(JoinState::Join) != nullptr) {
+        return 0;
+    }
+    if (m_currentPushNotificationState == PushNotificationState::Mute) {
+        return 0;
+    }
+    if (m_currentPushNotificationState == PushNotificationState::MentionKeyword || isLowPriority()) {
+        return int(highlightCount());
+    }
+    return int(notificationCount());
 }
 
 bool NeoChatRoom::hasFileUploading() const
@@ -220,6 +249,9 @@ void NeoChatRoom::acceptInvitation()
 
 void NeoChatRoom::forget()
 {
+    if (const auto &predecessor = dynamic_cast<NeoChatRoom *>(this->predecessor(JoinState::Join))) {
+        predecessor->forget();
+    }
     connection()->forgetRoom(id());
 }
 
@@ -228,7 +260,10 @@ QVariantList NeoChatRoom::getUsersTyping() const
     auto users = usersTyping();
     users.removeAll(localUser());
     QVariantList userVariants;
-    for (User *user : users) {
+    for (const auto &user : users) {
+        if (connection()->isIgnored(user->id())) {
+            continue;
+        }
         userVariants.append(QVariantMap{
             {"id"_ls, user->id()},
             {"avatarMediaId"_ls, user->avatarMediaId(this)},
@@ -502,9 +537,7 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
     }
 
     if (isThread) {
-        EventHandler eventHandler;
-        eventHandler.setRoom(this);
-        eventHandler.setEvent(&**replyIt);
+        EventHandler eventHandler(this, &**replyIt);
 
         const bool isFallingBack = !eventHandler.isThreaded();
 
@@ -551,9 +584,7 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
     if (isReply) {
         const auto &replyEvt = **replyIt;
 
-        EventHandler eventHandler;
-        eventHandler.setRoom(this);
-        eventHandler.setEvent(&**replyIt);
+        EventHandler eventHandler(this, &**replyIt);
 
         // clang-format off
         QJsonObject json{
@@ -1267,12 +1298,10 @@ void NeoChatRoom::removeParent(const QString &parentId)
     if (!currentState().contains("m.space.parent"_ls, parentId)) {
         return;
     }
-    if (auto parent = static_cast<NeoChatRoom *>(connection()->room(parentId))) {
-        setState("m.space.parent"_ls, parentId, {});
-    }
+    setState("m.space.parent"_ls, parentId, {});
 }
 
-bool NeoChatRoom::isSpace()
+bool NeoChatRoom::isSpace() const
 {
     const auto creationEvent = this->creation();
     if (!creationEvent) {
@@ -1282,7 +1311,23 @@ bool NeoChatRoom::isSpace()
     return creationEvent->roomType() == RoomType::Space;
 }
 
-void NeoChatRoom::addChild(const QString &childId, bool setChildParent, bool canonical, bool suggested)
+qsizetype NeoChatRoom::childrenNotificationCount()
+{
+    if (!isSpace()) {
+        return 0;
+    }
+    return SpaceHierarchyCache::instance().notificationCountForSpace(id());
+}
+
+bool NeoChatRoom::childrenHaveHighlightNotifications() const
+{
+    if (!isSpace()) {
+        return false;
+    }
+    return SpaceHierarchyCache::instance().spaceHasHighlightNotifications(id());
+}
+
+void NeoChatRoom::addChild(const QString &childId, bool setChildParent, bool canonical, bool suggested, const QString &order)
 {
     if (!isSpace()) {
         return;
@@ -1290,7 +1335,9 @@ void NeoChatRoom::addChild(const QString &childId, bool setChildParent, bool can
     if (!canSendEvent("m.space.child"_ls)) {
         return;
     }
-    setState("m.space.child"_ls, childId, QJsonObject{{QLatin1String("via"), QJsonArray{connection()->domain()}}, {"suggested"_ls, suggested}});
+    setState("m.space.child"_ls,
+             childId,
+             QJsonObject{{QLatin1String("via"), QJsonArray{connection()->domain()}}, {"suggested"_ls, suggested}, {"order"_ls, order}});
 
     if (setChildParent) {
         if (auto child = static_cast<NeoChatRoom *>(connection()->room(childId))) {
@@ -1356,6 +1403,28 @@ void NeoChatRoom::toggleChildSuggested(const QString &childId)
     }
 }
 
+void NeoChatRoom::setChildOrder(const QString &childId, const QString &order)
+{
+    if (!isSpace()) {
+        return;
+    }
+    if (!canSendEvent("m.space.child"_ls)) {
+        return;
+    }
+    if (const auto childEvent = currentState().get("m.space.child"_ls, childId)) {
+        auto content = childEvent->contentJson();
+        if (!content.contains("via"_ls)) {
+            return;
+        }
+        if (content.value("order"_ls).toString() == order) {
+            return;
+        }
+
+        content.insert("order"_ls, order);
+        setState("m.space.child"_ls, childId, content);
+    }
+}
+
 PushNotificationState::State NeoChatRoom::pushNotificationState() const
 {
     return m_currentPushNotificationState;
@@ -1826,7 +1895,11 @@ int NeoChatRoom::maxRoomVersion() const
 
 Quotient::User *NeoChatRoom::directChatRemoteUser() const
 {
-    return connection()->directChatUsers(this)[0];
+    auto users = connection()->directChatUsers(this);
+    if (users.isEmpty()) {
+        return nullptr;
+    }
+    return users[0];
 }
 
 void NeoChatRoom::sendLocation(float lat, float lon, const QString &description)
@@ -1906,4 +1979,9 @@ User *NeoChatRoom::invitingUser() const
     return connection()->user(currentState().get<RoomMemberEvent>(connection()->userId())->senderId());
 }
 
+void NeoChatRoom::setRoomState(const QString &type, const QString &stateKey, const QByteArray &content)
+{
+    setState(type, stateKey, QJsonDocument::fromJson(content).object());
+}
+
 #include "moc_neochatroom.cpp"
