@@ -57,7 +57,7 @@ void NotificationsManager::processNotificationJob(QPointer<NeoChatConnection> co
     if (job == nullptr) {
         return;
     }
-    if (connection == nullptr) {
+    if (connection == nullptr || !connection->isLoggedIn()) {
         qWarning() << QStringLiteral("No connection for GetNotificationsJob %1").arg(job->objectName());
         return;
     }
@@ -150,7 +150,7 @@ void NotificationsManager::processNotificationJob(QPointer<NeoChatConnection> co
 
 bool NotificationsManager::shouldPostNotification(QPointer<NeoChatConnection> connection, const QJsonValue &notification)
 {
-    if (connection == nullptr) {
+    if (connection == nullptr || !connection->isLoggedIn()) {
         return false;
     }
 
@@ -216,7 +216,7 @@ void NotificationsManager::postNotification(NeoChatRoom *room,
         auto connection = dynamic_cast<NeoChatConnection *>(Controller::instance().accounts().get(room->localUser()->id()));
         Controller::instance().setActiveConnection(connection);
         RoomManager::instance().setConnection(connection);
-        RoomManager::instance().enterRoom(room);
+        RoomManager::instance().resolveResource(room->id());
     });
 
     if (canReply) {
@@ -251,7 +251,7 @@ void NotificationsManager::postInviteNotification(NeoChatRoom *rawRoom, const QS
         }
         WindowController::instance().showAndRaiseWindow(notification->xdgActivationToken());
         notification->close();
-        RoomManager::instance().enterRoom(room);
+        RoomManager::instance().resolveResource(room->id());
     });
 
     const auto acceptAction = notification->addAction(i18nc("@action:button The thing being accepted is an invitation to chat", "Accept"));
